-- lua/gemini/api.lua

local M = {}

local function get_api_key()
	-- Check for environment variable
	local api_key = os.getenv("GEMINI_API_KEY")
	if api_key then
		return api_key
	end

	-- Check for Neovim global variable
	api_key = vim.g.gemini_api_key
	if api_key then
		return api_key
	end

	return nil -- API key not found
end

local function make_request(prompt)
	local api_key = get_api_key()

	if not api_key then
		vim.notify(
			"Google AI API key not set. Set GEMINI_API_KEY environment variable or g.gemini_api_key in your init.vim or init.lua.",
			vim.log.levels.ERROR
		)
		return nil
	end

	-- List models first to verify available options
	local list_models_command = string.format(
		"curl -s 'https://generativelanguage.googleapis.com/v1/models?key=%s'",
		api_key
	)
	
	local models_result = vim.fn.system(list_models_command)
	local success, models_data = pcall(vim.json.decode, models_result)
	
	if success and models_data.models then
		vim.notify("Available models: " .. vim.inspect(models_data.models), vim.log.levels.DEBUG)
	end

	local model = "gemini-1.0-pro" -- Updated model name
	local payload = vim.json.encode({
		contents = {
			{
				parts = {
					{
						text = prompt,
					},
				},
			},
		},
	})

	-- Escape the payload for shell
	payload = vim.fn.shellescape(payload)

	-- Updated API endpoint URL with v1 path
	local command = string.format(
		"curl -s -X POST "
			.. "'https://generativelanguage.googleapis.com/v1/models/%s:generateContent?key=%s' "
			.. "-H 'Content-Type: application/json' "
			.. "-d %s",
		model,
		api_key,
		payload
	)

	local result = vim.fn.system(command)

	-- Check for errors during the curl execution
	if vim.v.shell_error ~= 0 then
		vim.notify("Error executing curl. Check your command and ensure curl is installed.", vim.log.levels.ERROR)
		return nil
	end

	local success, decoded_result = pcall(vim.json.decode, result)
	if not success then
		vim.notify("Failed to decode API response: " .. result, vim.log.levels.ERROR)
		return nil
	end

	return decoded_result
end

function M.get_response(prompt)
	local result = make_request(prompt)

	if result then
		if result.error then
			vim.notify("API Error: " .. vim.inspect(result.error), vim.log.levels.ERROR)
			return nil
		end

		if
			result.candidates
			and result.candidates[1]
			and result.candidates[1].content
			and result.candidates[1].content.parts
			and result.candidates[1].content.parts[1]
			and result.candidates[1].content.parts[1].text
		then
			return result.candidates[1].content.parts[1].text
		end
		
		vim.notify("Unexpected response structure: " .. vim.inspect(result), vim.log.levels.ERROR)
	end
	
	vim.notify("No response from Google AI API or malformed response.", vim.log.levels.ERROR)
	return nil
end

return M
